import { Reducer } from "react";
import "ts-array-ext/findAndReplace";
/**
 * Different mutating types to be performed in the reducers dispatch.
 */
export declare enum ListReducerActionType {
    /** Remove one or many item(s) from state */
    Remove = 0,
    /** Replace one or many item(s) from state if present */
    Update = 1,
    /** Append one or many item(s) to state*/
    Add = 2,
    /** Replace or append one or many item(s) of state*/
    AddOrUpdate = 3,
    /** Reset the whole state */
    Reset = 4
}
declare type ActionDataType<T, K> = K extends ListReducerActionType.Reset ? T[] : K extends ListReducerActionType.Remove ? T[keyof T][] | T[keyof T] : T[] | T;
declare type ListReducerActionInput<T, K extends ListReducerActionType> = {
    type: K;
    data: ActionDataType<T, K>;
};
export declare type AllListActions<T> = ListReducerActionInput<T, ListReducerActionType.Remove> | ListReducerActionInput<T, ListReducerActionType.Update> | ListReducerActionInput<T, ListReducerActionType.Add> | ListReducerActionInput<T, ListReducerActionType.Reset> | ListReducerActionInput<T, ListReducerActionType.AddOrUpdate>;
declare const _default: <T = unknown>(key: keyof T) => Reducer<T[], AllListActions<T>>;
/**
 * @typeparam `T` type of the reducer state
 * @param {keyof T} key value of `U`
 * @return {Reducer} React reducer for a stateful list of `T`
 *
 * Can be initiated like this
 * `ListReducer<Entity>("id")`
 * Where `Entity` is the type of the list and `"id"` is a property key on the type
 * that is to be used to find index in the list
 */
export default _default;
