"use strict";
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ListReducerActionType = void 0;
// @ts-ignore
require("ts-array-ext/findAndReplace");
/**
 * Different mutating types to be performed in the reducers dispatch.
 */
var ListReducerActionType;
(function (ListReducerActionType) {
    /** Remove one or many item(s) from state */
    ListReducerActionType[ListReducerActionType["Remove"] = 0] = "Remove";
    /** Replace one or many item(s) from state if present */
    ListReducerActionType[ListReducerActionType["Update"] = 1] = "Update";
    /** Append one or many item(s) to state*/
    ListReducerActionType[ListReducerActionType["Add"] = 2] = "Add";
    /** Replace or append one or many item(s) of state*/
    ListReducerActionType[ListReducerActionType["AddOrUpdate"] = 3] = "AddOrUpdate";
    /** Reset the whole state */
    ListReducerActionType[ListReducerActionType["Reset"] = 4] = "Reset";
})(ListReducerActionType = exports.ListReducerActionType || (exports.ListReducerActionType = {}));
var isArray = function (t) {
    return Array.isArray(t);
};
/**
 * @typeparam `T` type of the reducer state
 * @param {keyof T} key value of `U`
 * @return {Reducer} React reducer for a stateful list of `T`
 *
 * Can be initiated like this
 * `ListReducer<Entity>("id")`
 * Where `Entity` is the type of the list and `"id"` is a property key on the type
 * that is to be used to find index in the list
 */
exports.default = (function (key) { return function (state, action) {
    switch (action.type) {
        case ListReducerActionType.AddOrUpdate:
            var dat_1 = action.data;
            if (isArray(dat_1)) {
                dat_1.map(function (d) { return state.findAndReplace(function (x) { return x[key] === d[key]; }, d, true); });
            }
            else {
                state.findAndReplace(function (x) { return x[key] === dat_1[key]; }, dat_1, true);
            }
            return __spreadArrays(state);
        case ListReducerActionType.Add:
            if (isArray(action.data)) {
                return __spreadArrays(state, action.data);
            }
            else {
                return __spreadArrays(state, [action.data]);
            }
        case ListReducerActionType.Update: {
            var dat_2 = action.data;
            if (isArray(dat_2)) {
                dat_2.forEach(function (d) {
                    state.findAndReplace(function (x) { return x[key] === d[key]; }, d);
                });
            }
            else {
                state.findAndReplace(function (x) { return x[key] === dat_2[key]; }, dat_2);
            }
            return __spreadArrays(state);
        }
        case ListReducerActionType.Remove: {
            var dat_3 = action.data;
            if (isArray(dat_3)) {
                return state.filter(function (t) { return dat_3.indexOf(t[key]) === -1; });
            }
            else {
                return state.filter(function (t) { return t[key] !== dat_3; });
            }
        }
        case ListReducerActionType.Reset:
            return action.data;
        default:
            return state;
    }
}; });
